@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -20,6 +20,7 @@
 */
 
 #include "gkrellmd.h"
+#include "gkrellmd-private.h"
 
 #ifndef HAVE_GETADDRINFO
 struct addrinfo
@@ -354,6 +355,11 @@ parse_config(gchar *config, gchar *arg)
 		_GK.max_clients = atoi(arg);
 	else if (!strcmp(config, "allow-host") || !strcmp(config, "a"))
 		allow_host_list = g_list_append(allow_host_list, g_strdup(arg));
+	else if (!strcmp(config, "plugin-enable") || !strcmp(config, "pe"))
+		gkrellmd_plugin_enable_list
+				= g_list_append(gkrellmd_plugin_enable_list, g_strdup(arg));
+	else if (!strcmp(config, "plugin") || !strcmp(config, "p"))
+		_GK.command_line_plugin = g_strdup(arg);
 	else if (!strcmp(config, "io-timeout"))
 		_GK.io_timeout = atoi(arg);
 	else if (!strcmp(config, "reconnect-timeout"))
@@ -416,7 +422,7 @@ load_config(gchar *path)
 static void
 read_config(void)
 	{
-	gchar	*path, *homedir;
+	gchar	*path;
 
 	_GK.update_HZ = 3;
 	_GK.max_clients = 2;
@@ -438,13 +444,13 @@ read_config(void)
 	g_free(path);
 #endif
 
-	homedir = (gchar *) g_get_home_dir();
-	if (!homedir)
-		homedir = ".";
+	_GK.homedir = (gchar *) g_get_home_dir();
+	if (!_GK.homedir)
+		_GK.homedir = ".";
 #if !defined(WIN32)
-	path = g_strdup_printf("%s/.%s", homedir, GKRELLMD_CONFIG);
+	path = g_strdup_printf("%s/.%s", _GK.homedir, GKRELLMD_CONFIG);
 #else
-	path = g_strdup_printf("%s/%s", homedir, GKRELLMD_CONFIG);
+	path = g_strdup_printf("%s/%s", _GK.homedir, GKRELLMD_CONFIG);
 #endif
 	load_config(path);
 	g_free(path);
@@ -469,6 +475,10 @@ usage(void)
 	printf(_("   -U, --user username       Change to this username after startup\n"));
 	printf(_("   -G, --group groupname     Change to this group after startup\n"));
 #endif
+	printf(_("   -p,  --plugin name        Enable a command line plugin\n"));
+	printf(_("   -pe, --plugin-enable name Enable an installed plugin\n"));
+	printf(_("       --plist               List plugins and exit\n"));
+	printf(_("       --plog                Print plugin install log\n"));
 	printf(_("   -V, --verbose\n"));
 	printf(_("   -v, --version\n"));
 	}
@@ -494,6 +504,16 @@ get_args(gint argc, gchar **argv)
 			_GK.verbose += 1;
 			continue;
 			}
+		if (!strcmp(opt, "plist"))
+			{
+			_GK.list_plugins = TRUE;
+			continue;
+			}
+		if (!strcmp(opt, "plog"))
+			{
+			_GK.log_plugins = TRUE;
+			continue;
+			}
 		else if (!strcmp(opt, "help") || !strcmp(opt, "h"))
 			{
 			usage();
@@ -759,7 +779,7 @@ main(gint argc, gchar **argv)
 
 	if (_GK.verbose)
 		printf("update_HZ=%d\n", _GK.update_HZ);
-	if (detach_flag)
+	if (detach_flag && !_GK.log_plugins && !_GK.list_plugins)
 		detach_from_terminal();
 	else
 		{
@@ -773,7 +793,10 @@ main(gint argc, gchar **argv)
 
 	gkrellm_sys_main_init();
 	drop_privileges();
+
+#if GLIB_CHECK_VERSION(2,0,0)
 	g_thread_init(NULL);
+#endif
 
 	_GK.start_time = time(0);
 	if (_GK.update_HZ < 1 || _GK.update_HZ > 10)
