@@ -1,5 +1,5 @@
 /*
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -482,7 +482,6 @@ update_battery_panel(Battery *bat)
 		gkrellm_draw_decal_pixmap(p, bat->power_decal, D_MISC_BATTERY);
 		}
 	draw_time_left_decal(bat, FALSE);
-
 	gkrellm_update_krell(p, bat->krell, bat->percent);
 	gkrellm_draw_panel_layers(p);
 	}
@@ -510,7 +509,7 @@ update_battery(void)
 				if (!bat->panel)
 					create_battery_panel(bat, TRUE);
 				if (bat->enabled)
-				update_battery_panel(bat);
+					update_battery_panel(bat);
 				}
 			}
 		seconds = (seconds + 1) % poll_interval;
@@ -1228,7 +1227,7 @@ static GkrellmMonitor	monitor_battery =
 	};
 
 GkrellmMonitor *
-init_battery_monitor(void)
+gkrellm_init_battery_monitor(void)
 	{
 	estimate_runtime[0] =  1.5;	/* 1.5 hour battery */
 	estimate_runtime[1] =	3.0;	/* 3 hour recharge */
