@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -123,9 +123,9 @@ client_cpu_line_from_server(gchar *line)
 	GList	*list;
 	Cpu		*cpu = NULL;
 	gint	n;
-	gulong	user, nice, sys, idle;
+	guint64	user, nice, sys, idle;
 
-	sscanf(line, "%d %lu %lu %lu %lu", &n, &user, &nice, &sys, &idle);
+	sscanf(line, "%d %llu %llu %llu %llu", &n, &user, &nice, &sys, &idle);
 	for (list = cpu_list; list; list = list->next)
 		{
 		cpu = (Cpu *) list->data;
@@ -367,9 +367,9 @@ client_net_line_from_server(gchar *line)
 	GList		*list;
 	NetData		*net;
 	gchar		name[32];
-	gulong		rx, tx;
+	guint64		rx, tx;
 
-	sscanf(line, "%31s %lu %lu", name, &rx, &tx);
+	sscanf(line, "%31s %llu %llu", name, &rx, &tx);
 	for (list = net_list; list; list = list->next)
 		{
 		net = (NetData *) list->data;
@@ -576,7 +576,7 @@ read_tcp_data(void)
 static void
 client_inet_reset(void)
 	{
-	free_glist_and_data(&inet_list);
+	gkrellm_free_glist_and_data(&inet_list);
 	}
 
 static void
@@ -1002,6 +1002,8 @@ typedef struct
 	gint		iodev;
 	gint		inter;
 
+	gint		group;
+
 	gchar		*vref;
 	gchar		*default_label;
 
@@ -1104,6 +1106,7 @@ client_sys_sensors_init(void)
 	{
 	GList			*list;
 	Sensor			*s;
+	gpointer		sr;
 
 	if (!sensors_list)
 		return;
@@ -1116,10 +1119,11 @@ client_sys_sensors_init(void)
 		/* The sysdep code in the server may be using the dir arg to get sensor
 		|  values, but dir is no longer needed.
 		*/
-		gkrellm_sensors_add_sensor(s->type, NULL, s->basename,
+		sr = gkrellm_sensors_add_sensor(s->type, NULL, s->basename,
 					s->id, s->iodev, s->inter,
 					s->factor, s->offset,
 					s->vref, s->default_label);
+		gkrellm_sensors_set_group(sr, s->group);
 		}
 	}
 
@@ -1133,10 +1137,11 @@ client_sensors_setup(gchar *line)
 		locale_fix(line);
 
 	s = g_new0(Sensor, 1);
+	s->group = SENSOR_GROUP_MAINBOARD;	/* Not in pre 2.2.0 versions */
 	if (sscanf(line,
-			"%d \"%127[^\"]\" %d %d %d %f %f \"%31[^\"]\" \"%31[^\"]\"",
+			"%d \"%127[^\"]\" %d %d %d %f %f \"%31[^\"]\" \"%31[^\"]\" %d",
 			&s->type, basename, &s->id, &s->iodev, &s->inter,
-			&s->factor, &s->offset, vref, default_label) == 9)
+			&s->factor, &s->offset, vref, default_label, &s->group) >= 9)
 		{
 		s->basename = g_strdup(basename);
 		if (strcmp(vref, "NONE"))
@@ -1202,12 +1207,14 @@ client_time_line_from_server(gchar *s)
 static void
 client_hostname_setup(gchar *s)
 	{
+	g_free(_GK.server_hostname);
 	_GK.server_hostname = g_strdup(s);
 	}
 
 static void
 client_sysname_setup(gchar *s)
 	{
+	g_free(_GK.server_sysname);
 	_GK.server_sysname = g_strdup(s);
 	}
 
@@ -1228,13 +1235,55 @@ KeyTable	monitor_table[] =
 	{"uptime",		client_sys_uptime_init },
 	};
 
+
 static gboolean	setup_done;		/* only one sys init */
 
+
+  /* Setup lines are received before monitor init functions are called, so
+  |  for plugins must save the strings until plugins are loaded.
+  */
+static GList	*client_plugin_setup_line_list;
+
+static void
+client_plugin_add_setup_line(gchar *line)
+	{
+	client_plugin_setup_line_list
+				= g_list_append(client_plugin_setup_line_list, g_strdup(line));
+	}
+
+  /* Plugins should call this in their gkrellm_init_plugin() function.
+  */
+void
+gkrellm_client_plugin_get_setup(gchar *key_name,
+				void (*setup_func_cb)(gchar *str))
+	{
+	GList	*list;
+	gchar	*line, *s;
+	gint	n;
+
+	if (!_GK.client_mode || !key_name || !setup_func_cb)
+		return;
+	for (list = client_plugin_setup_line_list; list; list = list->next)
+		{
+		line = (gchar *) list->data;
+		n = strlen(key_name);
+		s = line + n;
+		if (!strncmp(line, key_name, n) && *s == ' ')
+			{
+			while (*s == ' ')
+				++s;
+			(*setup_func_cb)(s);
+			}
+		}
+	}
+
 static void
 client_monitor_setup(gchar *line)
 	{
 	void			(*func)();
+	gchar			buf[64];
 	gint			i;
+	gboolean		found_builtin = FALSE;
 
 	if (!*line || setup_done)
 		return;
@@ -1244,9 +1293,18 @@ client_monitor_setup(gchar *line)
 			{
 			func = monitor_table[i].func;
 			(*func)();
+			found_builtin = TRUE;
 			break;
 			}
 		}
+	/* The client mode init work of a plugin must be defered since they
+	|  aren't loaded yet.  Set up so they will can get an "available" flag.
+	*/
+	if (!found_builtin)
+		{
+		snprintf(buf, sizeof(buf), "%s available", line);
+		client_plugin_add_setup_line(buf);
+		}
 	}
 
 static void
@@ -1291,6 +1349,7 @@ client_server_reconnect_timeout(gchar *line)
 		_GK.client_server_reconnect_timeout = 0;
 	}
 
+
 /* ================================================================= */
 KeyTable	setup_table[] =
 	{
@@ -1311,8 +1370,61 @@ KeyTable	setup_table[] =
 	{"<error>",			client_server_error },
 	{"<io_timeout>",	client_server_io_timeout},
 	{"<reconnect_timeout>", client_server_reconnect_timeout},
+
+	{"<plugin_setup>",	client_plugin_add_setup_line},
 	};
 
+
+typedef struct
+	{
+	GkrellmMonitor	*mon;
+	gchar			*key_name;
+	void			(*func_cb)(gchar *line);
+	}
+	ClientPlugin;
+
+static GList	*client_plugin_serve_data_list;
+
+static GkrellmFunc
+client_plugin_func(gchar *line)
+	{
+	GList			*list;
+	ClientPlugin	*plug;
+	void			(*func)() = NULL;
+	gchar			*s;
+	gint			n;
+
+	for (list = client_plugin_serve_data_list; list; list = list->next)
+		{
+		plug = (ClientPlugin *) list->data;
+		n = strlen(plug->key_name);
+		s = line + n;
+		if (*s == '>' && !strncmp(plug->key_name, line, n))
+			{
+			func = plug->func_cb;
+			break;
+			}
+		}
+	return func;
+	}
+
+void
+gkrellm_client_plugin_serve_data_connect(GkrellmMonitor *mon,
+			gchar *key_name, void (*func_cb)(gchar *line))
+	{
+	ClientPlugin	*plug;
+
+	if (!mon || !key_name || !func_cb)
+		return;
+	plug = g_new0(ClientPlugin, 1);
+	plug->mon = mon;
+	plug->key_name = g_strdup(key_name);
+	plug->func_cb = func_cb;
+	client_plugin_serve_data_list
+				= g_list_append(client_plugin_serve_data_list, plug);
+	}
+
+
 KeyTable	update_table[] =
 	{
 	{"<cpu>",			client_cpu_line_from_server},
@@ -1332,10 +1444,12 @@ KeyTable	update_table[] =
 	{"<sensors>",		client_sensors_line_from_server},
 	{"<time>",			client_time_line_from_server},
 	{"<uptime>",		client_uptime_line_from_server},
+
 //	{"<>",			client__line_from_server},
 	};
 
 
+
 static gint
 getline(gint fd, gchar *buf, gint len)
 	{
@@ -1398,6 +1512,8 @@ process_server_line(KeyTable *table, gint table_size, gchar *line)
 				break;
 				}
 			}
+		if (!func)
+			func = client_plugin_func(line + 1);
 		}
 	else if (func)
 		(*func)(line);
@@ -1419,6 +1535,9 @@ read_server_setup(gint fd)
 	locale_sync();
 	_GK.client_server_read_time = time(0);
 	table_size = sizeof(setup_table) / sizeof(KeyTable);
+
+	gkrellm_free_glist_and_data(&client_plugin_setup_line_list);
+
 	while (1)
 		{
 		getline(fd, buf, sizeof(buf));
@@ -1633,3 +1752,10 @@ gkrellm_client_mode_connect_thread(void)
 	client_mode_thread_busy = TRUE;
 	g_thread_create(client_mode_connect_thread, NULL, FALSE, NULL);
 	}
+
+
+gboolean
+gkrellm_client_mode(void)
+	{
+	return _GK.client_mode;
+	}
