@@ -231,15 +231,22 @@ update_clock(void)
 	{
 	struct tm		*ptm;
 	gboolean		update_minutes = FALSE;
-	static gint		min_prev, hour_prev = -1;
+	static gint		min_prev, hour_prev = -1, sec_prev = -1;
 	ChimeData		*chime;
 
-	if (!GK.second_tick)
-		return;
 	if (_GK.client_mode)
+		{
 		ptm = gkrellm_client_server_time();
+		if (sec_prev == ptm->tm_sec)
+			return;
+		sec_prev = ptm->tm_sec;
+		}
 	else
+		{
+		if (!GK.second_tick)
+			return;
 		ptm = &gkrellm_current_tm;
+		}
 
 	if (   ptm->tm_min != min_prev
 		|| ptm->tm_hour != hour_prev
