@@ -393,7 +393,7 @@ assign_style_entry(GkrellmStyle *style, gchar *value, gint entry_flag)
 	{
 	if (entry_flag == GKRELLMSTYLE_KRELL_YOFF)
 		style->krell_yoff = atoi(value);
-	if (entry_flag == GKRELLMSTYLE_KRELL_YOFF_NOT_SCALABLE)
+	else if (entry_flag == GKRELLMSTYLE_KRELL_YOFF_NOT_SCALABLE)
 		style->krell_yoff_not_scalable = parse_boolean(value);
 	else if (entry_flag == GKRELLMSTYLE_KRELL_EXPAND)
 		gkrellm_set_krell_expand(style, value);
@@ -416,6 +416,8 @@ assign_style_entry(GkrellmStyle *style, gchar *value, gint entry_flag)
 		else
 			style->label_position = GKRELLM_LABEL_NONE;
 		}
+	else if (entry_flag == GKRELLMSTYLE_LABEL_YOFF)
+		style->label_yoff = atoi(value) * _GK.theme_scale / 100;
 	else if (entry_flag == OLD_SET_MARGIN)	/* Deprecated as of 1.2.9 */
 		{
 		style->margin.left = atoi(value) * _GK.theme_scale / 100;
@@ -567,6 +569,7 @@ static struct string_map
 	{ "krell_left_margin",	GKRELLMSTYLE_KRELL_LEFT_MARGIN },
 	{ "krell_right_margin",	GKRELLMSTYLE_KRELL_RIGHT_MARGIN },
 	{ "label_position",		GKRELLMSTYLE_LABEL_POSITION },
+	{ "label_yoff",			GKRELLMSTYLE_LABEL_YOFF },
 	{ "margins",			SET_ALL_MARGINS },
 	{ "left_margin",		GKRELLMSTYLE_LEFT_MARGIN },
 	{ "right_margin",		GKRELLMSTYLE_RIGHT_MARGIN },
@@ -1447,6 +1450,8 @@ static struct	_config
 	{"chart_out_color",		NULL,		&_GK.chart_out_color,		-100 },
 	{"chart_out_color_grid",NULL,		&_GK.chart_out_color_grid,	-100 },
 
+	{"chart_text_no_fill",	&_GK.chart_text_no_fill,	NULL,			0  },
+
 	{"bg_grid_mode",		&_GK.bg_grid_mode,		NULL,			0  },
 
 	{"frame_top_border",	NULL,		&frame_top_border,			-100 },
@@ -1849,6 +1854,7 @@ gkrellm_init_theme(void)
 	_GK.chart_height_min = 5;
 	_GK.chart_height_max = 200;
 	_GK.chart_width_ref = 60;
+	_GK.chart_text_no_fill = FALSE;
 	_GK.bg_separator_height = 2;
 	_GK.allow_scaling = FALSE;
 	_GK.bg_grid_mode = GRID_MODE_NORMAL;
