@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -88,8 +88,8 @@
 /* -------------------------------------------------------------------
 */
 #define	GKRELLM_VERSION_MAJOR	2
-#define	GKRELLM_VERSION_MINOR	1
-#define	GKRELLM_VERSION_REV		24
+#define	GKRELLM_VERSION_MINOR	2
+#define	GKRELLM_VERSION_REV		4
 #define	GKRELLM_EXTRAVERSION	""
 
 #define GKRELLM_CHECK_VERSION(major,minor,rev)    \
@@ -142,10 +142,14 @@
 
 #define	CFG_BUFSIZE		384
 
+
   /* Features
   */
-#define	GKRELLM_HAVE_THEME_SCALE		1
-#define	GKRELLM_HAVE_DECAL_TEXT_INSERT	1
+#define	GKRELLM_HAVE_THEME_SCALE			1
+#define	GKRELLM_HAVE_DECAL_TEXT_INSERT		1
+#define	GKRELLM_HAVE_CLIENT_MODE_PLUGINS 	1
+#define	GKRELLM_USING_PANGO					1
+#define	GKRELLM_HAVE_DECAL_SCROLL_TEXT		1
 
 
   /* Label midpoints are positioned as a percent of chart_width.
@@ -203,14 +207,17 @@ typedef struct
 #define SENSOR_FAN          1
 #define SENSOR_VOLTAGE      2
 
+#define	SENSOR_GROUP_MAINBOARD	0
+#define	SENSOR_GROUP_DISK		1
+
 
 #define	FULL_SCALE_GRIDS		5
 
 
 typedef struct
 	{
-	GdkFont		*font;
-	GdkFont		**font_seed;
+	PangoFontDescription *font;
+	PangoFontDescription **font_seed;
 	gpointer	privat;
 	gint		effect;
 	gint		internal;
@@ -220,20 +227,35 @@ typedef struct
 	}
 	GkrellmTextstyle;
 
-
+typedef struct
+	{
+	gchar				*text;
+	gint				x_off,
+						y_off;
+	gint				x_off_prev;		/* Kludge to check if d->x_off was */
+										/* modified directly.		*/
+	gpointer			privat;
+	GkrellmTextstyle	text_style;
+	}
+	GkrellmText;
 
   /* Values for GkrellmDecal flags
   */
-#define DF_TOP_LAYER		0x1
-#define	DF_MOVED			0x2
-#define DF_LOCAL_PIXMAPS	0x4
-#define	DF_OVERLAY_PIXMAPS	0x8
+#define DF_TOP_LAYER			0x1
+#define	DF_MOVED				0x2
+#define DF_LOCAL_PIXMAPS		0x4
+#define	DF_OVERLAY_PIXMAPS		0x8
+#define	DF_TEXT_OVERLAPS		0x10
+#define	DF_SCROLL_TEXT_DIVERTED	0x20
+#define	DF_SCROLL_TEXT_CENTER	0x40
+#define	DF_SCROLL_TEXT_H_LOOP	0x80
+#define	DF_SCROLL_TEXT_V_LOOP	0x100
+#define	DF_TEXT_USE_MARKUP		0x200
 
   /* Values for GkrellmDecal state
   */
 #define	DS_INVISIBLE	0
-#define	DS_ERASING		1
-#define	DS_VISIBLE		2
+#define	DS_VISIBLE		1
 
   /* A decal is a pixmap or a part of a pixmap drawn on a panel.  The
   |  pixmap can be a graphic, a vertical stack of graphics, or a drawn
@@ -252,10 +274,18 @@ typedef struct
 	gint		value;		/* Index into stack, text value, etc	*/
 	gboolean	modified;
 	gint		x_off,
-				y_baseline;
+				y_ink;			/* Pixels from top of Pango glyph to 1st ink*/
 	gint		x_old, y_old;	/* Used if decal was moved */
 	GkrellmTextstyle text_style;	/* Used if decal is a drawn text string	*/
 	gpointer	panel;
+	GList		*text_list;
+	gchar		*scroll_text;
+	gint		scroll_width,
+				scroll_height,
+				y_off;
+
+	guint		chart_sequence_id;	/* For drawing text decal on a chart */
+	GdkRectangle ink;
 	}
 	GkrellmDecal;
 
@@ -269,15 +299,15 @@ typedef struct
 	{
 	gchar		*string;
 	gpointer	privat;
-	gint		lbearing,
-				rbearing,
+	gint		old1,		/* olds are leftovers from GdkFont->Pango */
+				old2,
 				width,
-				ascent,
-				descent;
+				height,
+				old3;
 	gint		position;		/* 0 - 100 %	*/
-	gint		x_panel;		/* x position of label start in panel */
-	gint		y_baseline;
-	gint		h_panel;
+	gint		x_panel,		/* x position of label start in panel */
+				y_panel,
+				h_panel;
 	}
 	GkrellmLabel;
 
@@ -342,6 +372,9 @@ typedef struct
 #define	GKRELLMSTYLE_BORDER				0x10000
 #define	GKRELLMSTYLE_TRANSPARENCY		0x20000
 #define	GKRELLMSTYLE_KRELL_YOFF_NOT_SCALABLE 0x40000
+#define	GKRELLMSTYLE_SCROLL_TEXT_CACHE_OFF	 0x80000
+#define	GKRELLMSTYLE_LABEL_YOFF			0x100000
+
 
   /* Some of these style entries do not apply to all monitors.
   */
@@ -358,9 +391,10 @@ typedef struct
 	gint		krell_left_margin,
 				krell_right_margin;
 	gboolean	krell_yoff_not_scalable;
-	gint		krell_spare;
-	gint		label_position,
-				spare0, spare1;
+	gint		label_yoff;
+	gint		label_position;
+	gboolean	scroll_text_cache_off;
+	gint		spare0;
 
 	gboolean	transparency;
 	gint		themed;			/* Non zero if theme has custom assignments */
@@ -437,7 +471,7 @@ typedef struct
 typedef struct
 	{
 	GtkWidget	*hbox;			/* Container box this area is packed into */
-	GtkWidget	*fixed;
+	GdkPixmap	*bg_text_layer_pixmap;  /* A bg_pixmap with text decals */
 	GtkWidget	*drawing_area;
 	GdkPixmap	*pixmap;		/* Expose pixmap */
 	GdkBitmap	*bg_mask;
@@ -456,11 +490,15 @@ typedef struct
 	gint		x, y, w, h;
 	gint		h_configure;
 
+	PangoLayout	*layout;
+
 	gint		transparency;
 	gboolean	modified;		/* True if decal/krell modified.	*/
 	gboolean	keep_lists;
 	gboolean	shown;
+	gboolean	need_decal_overlap_check;
 	gboolean	label_on_top_of_decals;
+	gboolean	scroll_text_cache_off;
 	gboolean	bg_piximage_override;
 	gboolean	button_signals_connected;
 	gint		id_press,		/* Signals if buttons in this panel	*/
@@ -472,6 +510,14 @@ typedef struct
 	GkrellmPanel;
 
 
+typedef struct
+	{
+	void		(*func)();
+	gpointer	data;
+	}
+	GkrellmCallback;
+
+
 #define CHART_WIDTH_MAX	1000
 #define CHART_WIDTH_MIN	25
 
@@ -559,6 +605,17 @@ typedef struct
 	}
 	GkrellmSpacer;
 
+typedef struct
+	{
+	gchar			*text;
+	gint			x, y;
+	gint			w, h;
+	gint			baseline,
+					y_ink;
+	gboolean		cache_valid;
+	}
+	GkrellmTextRun;
+
 typedef struct
 	{
 	GtkWidget	*box;			/* Container box this chart is packed into */
@@ -601,10 +658,20 @@ typedef struct
 	GdkPixmap	*bg_pixmap,			/* Working bg with grid lines	*/
 				*bg_src_pixmap,		/* bg with no grid lines		*/
 				*bg_grid_pixmap,	/* This + bg_src_pixmap builds bg_pixmap*/
-				*bg_clean_pixmap;	/* This + trans builds bg_src_pixmap */
+				*bg_clean_pixmap,	/* This + trans builds bg_src_pixmap */
+				*bg_text_pixmap;	/* Pango rendered chart text layer */
 	GdkBitmap	*bg_mask;
 	gint		transparency;
 	gboolean	bg_piximage_override;
+	guint		bg_sequence_id;
+
+	gchar		*text_format_string;
+	GList		*text_run_list;		/* Caching chart text runs */
+	gint		h_text,
+				y_ink,
+				baseline_ref;
+	guint		text_run_sequence_id;
+	gboolean	text_format_reuse;
 
 	gint		y_mapped,
 				auto_recalibrate_delay;
@@ -656,6 +723,7 @@ typedef struct
 
 	gint		y_p,
 				y_pp;
+	gboolean	y_p_valid;
 	gint		y,				/* Each data layer draws at an offset y and */
 				h,				/* within height h of its parent GkrellmChart*/
 				w;				/* Width of data allocated.				*/
