@@ -80,6 +80,19 @@ setup_panel_style(GkrellmPanel *p, GkrellmStyle *style)
 		}
 	}
 
+void
+gkrellm_panel_label_get_position(GkrellmStyle *style, gint *x_position,
+				gint *y_off)
+	{
+	if (!style)
+		return;
+	if (x_position)
+		*x_position = style->label_position;
+	if (y_off)
+		*y_off = style->label_yoff;
+	}
+
+
   /* Calculate panel height required for given border, label height,
   |  krell heights, and decal heights.  Also calculate label extents
   |  and load all this info into a GkrellmLabel structure.
@@ -126,7 +139,12 @@ gkrellm_panel_configure(GkrellmPanel *p, gchar *string, GkrellmStyle *style)
 		lbl->height = 0;
 		baseline = 0;
 		}
-	h_panel = top_margin + bottom_margin;
+
+	if (style && style->label_yoff > 0)
+		h_panel = style->label_yoff + bottom_margin;
+	else
+		h_panel = top_margin + bottom_margin;
+
 	h_panel += lbl->height + (ts ? ts->effect : 0);
 
 	/* If krell_yoff is -1, then in gkrellm_create_krell() k->y0 was put
@@ -173,7 +191,10 @@ gkrellm_panel_configure(GkrellmPanel *p, gchar *string, GkrellmStyle *style)
 	if (h_panel <= 0)
 		h_panel = 1;
 
-	lbl->y_panel = top_margin - (baseline - lbl->height);
+	if (style && style->label_yoff > 0)
+		lbl->y_panel = style->label_yoff - (baseline - lbl->height);
+	else
+		lbl->y_panel = top_margin - (baseline - lbl->height);
 	p->h_configure = h_panel;
 	}
 
@@ -193,7 +214,8 @@ draw_panel_label(GkrellmPanel *p, gboolean to_bg)
 		{
 		m = gkrellm_get_style_margins(p->style);
 		lbl->width = gkrellm_gdk_string_width(ts->font, s) + ts->effect;
-		xdst = gkrellm_label_position(lbl->position, p->w, lbl->width, m->left);
+		xdst = gkrellm_label_x_position(lbl->position, p->w,
+					lbl->width, m->left);
 		lbl->x_panel = xdst;
 		gkrellm_draw_string(p->pixmap, ts, xdst, lbl->y_panel, s);
 		if (to_bg)
