@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -113,6 +113,8 @@ static gint		fork_scaling = 1;
 static gchar	*text_format,
 				*text_format_locale;
 
+static gboolean	new_text_format = TRUE;
+
 static void
 format_proc_data(ProcMon *p, gchar *src_string, gchar *buf, gint size)
 	{
@@ -170,6 +172,9 @@ draw_proc_extra(void)
 	if (!proc.chart || !proc.extra_info)
 		return;
 	format_proc_data(&proc, text_format_locale, buf, sizeof(buf));
+	if (!new_text_format)
+		gkrellm_chart_reuse_text_format(proc.chart);
+	new_text_format = FALSE;
 	gkrellm_draw_chart_text(proc.chart, style_id, buf);
 	}
 
@@ -566,6 +571,7 @@ create_proc_monitor(GtkWidget *vbox, gint first_create)
 	if (proc.sensor_decal)
 		adjust_sensors_display(TRUE);
 
+	new_text_format = TRUE;
 	if (first_create)
 		{
 		g_signal_connect(G_OBJECT(cp->drawing_area), "expose_event",
@@ -710,7 +716,10 @@ load_proc_config(gchar *arg)
 		else if (!strcmp(config, "sensor_mode"))
 			sscanf(item, "%d\n", &sensor_separate_mode);
 		else if (!strcmp(config, "text_format"))
+			{
 			gkrellm_locale_dup_string(&text_format, item, &text_format_locale);
+			new_text_format = TRUE;
+			}
 		else if (!strcmp(config, GKRELLM_CHARTCONFIG_KEYWORD))
 			gkrellm_load_chartconfig(&proc.chart_config, item, 2);
 		else if (!strcmp(config, GKRELLM_ALERTCONFIG_KEYWORD))
@@ -797,6 +806,7 @@ cb_text_format(GtkWidget *widget, gpointer data)
 
 	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(text_format_combo)->entry));
 	gkrellm_locale_dup_string(&text_format, s, &text_format_locale);
+	new_text_format = TRUE;
 	refresh_proc_chart(proc.chart);
 	}
 
@@ -946,6 +956,7 @@ create_proc_tab(GtkWidget *tab_vbox)
 	list = g_list_append(list,
 			_("\\f$L\\r\\f$F \\w88\\b\\p\\a$p\\f procs\\n\\e$u\\f users"));
 	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
+	gtk_combo_set_case_sensitive(GTK_COMBO(text_format_combo), TRUE);
 	g_list_free(list);
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
 			text_format);
@@ -1008,7 +1019,7 @@ static GkrellmMonitor	monitor_proc =
 	};
 
 GkrellmMonitor *
-init_proc_monitor(void)
+gkrellm_init_proc_monitor(void)
 	{
 	GkrellmChartconfig	*cf;
 
