@@ -609,6 +609,8 @@ gkrellm_sensor_read_voltage(gpointer sr, gfloat *voltage)
 /* =================================================================== */
 /* The sensors monitor */
 
+static void		sensor_reset_optionmenu(Sensor *sensor);
+
 #define	SENSOR_STYLE_NAME	"sensors"
 
 /* Temperature and fan sensors can be located on different panels depending
@@ -682,6 +684,9 @@ static gint		minus_width;		/* If will be drawing neg voltages */
   /* If drawing '-' sign, grub a pixel or two to tighten the layout */
 static gint		pixel_grub;
 
+
+static gint		sensor_config_version;
+
   /* Avoid writing decimal values into the config to avoid possible
   |  locale changing decimal point breakage (decimal point can be '.' or ',')
   */
@@ -689,6 +694,37 @@ static gint		pixel_grub;
 static gfloat		sensor_float_factor = 1.0,
 					gkrellm_float_factor = 1.0;
 
+void
+gkrellm_sensor_reset_location(gpointer sr)
+	{
+	GList	*list;
+	Sensor	*sensor;
+
+	for (list = sensor_list; list; list = list->next)
+		{
+		sensor = (Sensor *) list->data;
+		if (sr == sensor)
+			{
+			sensor->location = SENSOR_PANEL_LOCATION;
+			sensor_reset_optionmenu(sensor);
+			break;
+			}
+		}
+	}
+
+
+static void
+sensor_relocation_error(gchar *pname)
+	{
+	gchar	*msg;
+
+	msg = g_strdup_printf(
+				_("Can't find a %s panel to relocate sensor to."),
+				pname ? pname : "?");
+	gkrellm_config_message_dialog(NULL, msg);
+	g_free(msg);
+	}
+
 static void
 add_sensor_monitor(Sensor *sr, GkrellmPanel **p, GList **smon_list)
 	{
@@ -979,17 +1015,28 @@ layout_volt_decals(GkrellmPanel *p, GkrellmStyle *style)
 static void
 update_disk_temperatures(void)
 	{
-	GList	*list;
-	Sensor	*sr;
-	gfloat	t;
-	gchar	units;
+	GList		*list;
+	Sensor		*sr;
+	gfloat		t;
+	gchar		units;
+	gboolean	display_failed = FALSE;
 
 	for (list = disk_temperature_list; list; list = list->next)
 		{
 		sr = (Sensor *) list->data;
 		sensor_read_temperature(sr, &t, &units);
-		gkrellm_disk_temperature_display((gpointer) sr, sr->id_name, t, units);
+		if (!gkrellm_disk_temperature_display((gpointer) sr, sr->id_name,
+					t, units))
+			{
+			/* disk panel was disabled, so put temp back on sensors panel
+			*/
+			display_failed = TRUE;
+			sr->location = SENSOR_PANEL_LOCATION;
+			sensor_reset_optionmenu(sr);
+			}
 		}
+	if (display_failed)
+		gkrellm_sensors_rebuild(TRUE, FALSE, FALSE);
 	}
 
 
@@ -1383,8 +1430,8 @@ create_sensors_monitor(gboolean do_temp, gboolean do_fan, gboolean do_volt,
 		gkrellm_spacers_hide(mon_sensors);
 	}
 
-static void
-rebuild_sensors_monitor(gboolean do_temp, gboolean do_fan, gboolean do_volt)
+void
+gkrellm_sensors_rebuild(gboolean do_temp, gboolean do_fan, gboolean do_volt)
 	{
 	destroy_sensors_monitor(do_temp, do_fan, do_volt);
 	create_sensors_monitor(do_temp, do_fan, do_volt, TRUE);
@@ -1487,15 +1534,20 @@ gkrellm_init_sensor_monitor(void)
   |  change to sensor-config.  But sensor_config is forward compatible
   |  since the float factor defaults to 1.0.
   */
+  /* gkrellm 2.2.3 sets sensor_config_version to 1 to allow sensor relocation
+  |  to composite CPU on a SMP machine.
+  */
+#define	SENSOR_CONFIG_VERSION		1
+
 #define	SENSOR_CONFIG_KEYWORD		"sensor"
 #define	SENSOR_CONFIG_FILE			"sensor-config"
 #define	SENSOR_2_1_14_CONFIG_FILE	"sensors_config"
 
+static void		cb_alert_config(GkrellmAlert *ap, Sensor *sr);
+
 static void
 create_sensor_alert(Sensor *s)
 	{
-	static void	cb_alert_config(GkrellmAlert *ap, Sensor *sr);
-
 	if (s->type == SENSOR_VOLTAGE)
 		s->alert = gkrellm_alert_create(NULL, s->name,
 				_("Sensor Volt Limits"),
@@ -1533,6 +1585,8 @@ save_sensors_config(FILE *f_not_used)
 	if (!f)
 		return;
 
+	fprintf(f, "%s sensor_config_version %d\n",
+				SENSOR_CONFIG_KEYWORD, SENSOR_CONFIG_VERSION);
 	fprintf(f, "%s sensor_float_factor %.0f\n",
 				SENSOR_CONFIG_KEYWORD, SENSOR_FLOAT_FACTOR);
 	fprintf(f, "%s gkrellm_float_factor %.0f\n",
@@ -1586,7 +1640,9 @@ load_sensors_config(gchar *arg)
 		return;
 	if (_GK.debug_level & DEBUG_SENSORS)
 		printf("load_sensors_config: <%s> <%s>\n", config, item);
-	if (!strcmp(config, "units_fahrenheit"))
+	if (!strcmp(config, "sensor_config_version"))
+		sscanf(item, "%d", &sensor_config_version);
+	else if (!strcmp(config, "units_fahrenheit"))
 		sscanf(item, "%d", &units_fahrenheit);
 	else if (!strcmp(config, "volt_display_mode"))
 		sscanf(item, "%d", &display_mode);
@@ -1627,7 +1683,21 @@ load_sensors_config(gchar *arg)
 		s->factor = f / sensor_float_factor;
 		s->offset = o / sensor_float_factor;
 		s->enabled = e;
-		s->location = (s->type == SENSOR_VOLTAGE) ? 0 : location;
+
+		if (s->type == SENSOR_VOLTAGE)
+			s->location = 0;
+		else
+			{
+			s->location = location;
+			if (   sensor_config_version == 0 && gkrellm_smp_cpus() > 0
+				&& location > PROC_PANEL_LOCATION
+			   )
+				/* gkrellm < 2.2.3 did not allow relocating to composite
+				|  CPU if on a SMP machine.  But with hyperthreading, user
+				|  may want to do this.
+				*/
+				s->location += 1;
+			}
 		if (!using_new_config && s->type != SENSOR_VOLTAGE)
 			s->enabled = TRUE;		/* Old config enabled with a label */
 		}
@@ -1705,9 +1775,12 @@ static GtkWidget	*alert_button;
 
 static Sensor		*dragged_sensor;
 
+static gint			sensor_last_group;
+
 static gboolean		(*original_row_drop_possible)();
 
 
+
 static void
 set_tree_store_model_data(GtkTreeStore *tree, GtkTreeIter *iter, Sensor *s)
 	{
@@ -1963,20 +2036,33 @@ cb_drag_end(GtkWidget *widget, GdkDragContext *context, gpointer data)
 	dragged_sensor = NULL;
 
 	if (type < 0)
-		rebuild_sensors_monitor(DO_TEMP, DO_FAN, DO_VOLT);
+		gkrellm_sensors_rebuild(DO_TEMP, DO_FAN, DO_VOLT);
 	else
-		rebuild_sensors_monitor(type == SENSOR_TEMPERATURE,
+		gkrellm_sensors_rebuild(type == SENSOR_TEMPERATURE,
 				type == SENSOR_FAN, type == SENSOR_VOLTAGE);
 
 	return FALSE;
 	}
 
+static void
+sensor_reset_optionmenu(Sensor *sensor)
+	{
+	Sensor	*sr;
+
+	if (!optionmenu)
+		return;
+	sr = get_referenced_sensor();
+	if (sr == sensor)
+		gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu),
+					SENSOR_PANEL_LOCATION);
+	}
+
 static void
 cb_location_menu(GtkOptionMenu *om, gpointer data)
 	{
 	GList		*list;
 	Sensor		*sr, *s;
-	gchar		*msg, *pname = NULL;
+	gchar		*pname = NULL;
 	gint		location;
 
 	location = gtk_option_menu_get_history(om);
@@ -2031,17 +2117,13 @@ cb_location_menu(GtkOptionMenu *om, gpointer data)
 			   )
 				s->location = SENSOR_PANEL_LOCATION;	/* is being replaced */
 			}
-	rebuild_sensors_monitor(DO_TEMP, DO_FAN, FALSE);
+	gkrellm_sensors_rebuild(DO_TEMP, DO_FAN, FALSE);
 
 	if (sr->location != location)	/* location failed */
 		{
 		gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu),
 					SENSOR_PANEL_LOCATION);
-		msg = g_strdup_printf(
-					_("Can't find a %s panel to relocate sensor to."),
-					pname ? pname : "?");
-		gkrellm_config_message_dialog(NULL, msg);
-		g_free(msg);
+		sensor_relocation_error(pname);
 		}
 	}
 
@@ -2052,11 +2134,11 @@ create_location_menu(gint group)
 	GtkWidget	*menu;
 	GtkWidget	*menuitem;
 	gint		n, n_cpus;
-	static gint	sig_id, last_group = -1;
+	static gint	sig_id;
 
-	if (group == last_group)
+	if (group == sensor_last_group)
 		return;
-	last_group = group;
+	sensor_last_group = group;
 
 	if (optionmenu_menu)
 		{
@@ -2077,9 +2159,7 @@ create_location_menu(gint group)
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 		gtk_widget_show(menuitem);
 
-		n_cpus = gkrellm_smp_cpus();
-		if (n_cpus == 0)
-			n_cpus = 1;
+		n_cpus = gkrellm_smp_cpus() + 1;
 		for (n = 0; n < n_cpus; ++n)
 			{
 			menuitem = gtk_menu_item_new_with_label(
@@ -2223,7 +2303,7 @@ cb_label_edited(GtkCellRendererText *cell, gchar *path_string,
 
 	if (gkrellm_locale_dup_string(&s->name, new_label, &s->name_locale))
 		{
-		rebuild_sensors_monitor(s->type == SENSOR_TEMPERATURE,
+		gkrellm_sensors_rebuild(s->type == SENSOR_TEMPERATURE,
 				s->type == SENSOR_FAN, s->type == SENSOR_VOLTAGE);
 		if (s->alert)
 			{
@@ -2258,7 +2338,7 @@ cb_enable(GtkCellRendererText *cell, gchar *path_string, gpointer data)
     change_row_reference(model, path);
 	gtk_tree_path_free(path);
 
-	rebuild_sensors_monitor(s->type == SENSOR_TEMPERATURE,
+	gkrellm_sensors_rebuild(s->type == SENSOR_TEMPERATURE,
 				s->type == SENSOR_FAN, s->type == SENSOR_VOLTAGE);
 
 	set_sensor_widget_states(s);
@@ -2302,7 +2382,7 @@ cb_temperature_units(GtkWidget *button, gpointer data)
 	for (list = sensor_list; list; list = list->next)
 		fix_temp_alert((Sensor *) list->data);
 
-	rebuild_sensors_monitor(DO_TEMP, FALSE, FALSE);
+	gkrellm_sensors_rebuild(DO_TEMP, FALSE, FALSE);
 	gkrellm_cpu_draw_sensors(NULL);
 	gkrellm_proc_draw_sensors(NULL);
 	}
@@ -2315,7 +2395,7 @@ cb_voltages_display(GtkWidget *entry, gpointer data)
 	for (i = 0; i < N_DISPLAY_MODES; ++i)
 		if (GTK_TOGGLE_BUTTON(display_mode_button[i])->active)
 			display_mode = i;
-	rebuild_sensors_monitor(FALSE, FALSE, DO_VOLT);
+	gkrellm_sensors_rebuild(FALSE, FALSE, DO_VOLT);
 	}
 
 static void
@@ -2361,6 +2441,12 @@ N_("\t$s    current sensor value.\n"),
 N_("\t$l    sensor label.\n"),
 	};
 
+static void
+sensors_tab_destroy(GtkWidget *w, gpointer data)
+	{
+	optionmenu = NULL;
+	}
+
 static void
 create_sensors_tab(GtkWidget *tab_vbox)
 	{
@@ -2376,10 +2462,15 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	gint					i;
 
 	row_reference = NULL;
+	sensor_last_group = -1;
+
 	tabs = gtk_notebook_new();
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
 	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);
 
+	g_signal_connect(GTK_OBJECT(tabs), "destroy",
+				G_CALLBACK(sensors_tab_destroy), NULL);
+
 /* --Setup tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
 	hbox = gtk_hbox_new(FALSE, 2);
