@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -1104,6 +1104,11 @@ gkrellm_sys_mem_read_data(void)
 	gboolean	using_tagged = FALSE;
 	guint64		total, used, x_used, free, shared, buffers, cached;
 
+	/* Default  0, so we don't get arbitrary values if not found,
+	|  e.g. MemShared is not present in 2.6.
+	*/
+	total = used = x_used = free = shared = buffers = cached = 0LL;
+
 	if ((f = fopen(PROC_MEMINFO_FILE, "r")) == NULL)
 		return;
 	while ((fgets(buf, sizeof(buf), f)) != NULL)
@@ -1344,8 +1349,8 @@ gkrellm_sys_fs_get_fsusage(gpointer fs, gchar *dir)
 
 	if (!statfs(dir, &st))
 		gkrellm_fs_assign_fsusage_data(fs,
-					(gulong) st.f_blocks, (gulong) st.f_bavail,
-					(gulong) st.f_bfree, (gulong) st.f_bsize);
+					(glong) st.f_blocks, (glong) st.f_bavail,
+					(glong) st.f_bfree, (glong) st.f_bsize);
 	else
 		gkrellm_fs_assign_fsusage_data(fs, 0, 0, 0, 0);
 	}
@@ -1398,10 +1403,11 @@ gkrellm_sys_fs_init(void)
 typedef struct
 	{
 	gint		id;
-	gboolean	got_info;
 	gchar		*info,
 				*state;
 	gint		full_cap;
+	gboolean	got_full_cap,
+				full_cap_bug;
 	}
 	BatteryFile;
 
@@ -1434,6 +1440,8 @@ setup_acpi_battery(gchar *bat)
 	static gint	id;
 
 	info = g_strconcat(ACPI_BATTERY_DIR, bat, "/info", NULL);
+	if (_GK.debug_level & DEBUG_BATTERY)
+		printf("setup_acpi_battery: %s\n", info);
 	if (!access(info, F_OK))
 		{
 		bf = g_new0(BatteryFile, 1);
@@ -1443,6 +1451,9 @@ setup_acpi_battery(gchar *bat)
 		if (!bf->state)
 			bf->state = get_acpi_battery_file(ACPI_BATTERY_DIR, bat, "status");
 		acpi_battery_list = g_list_append(acpi_battery_list, bf);
+		if (_GK.debug_level & DEBUG_BATTERY)
+			printf("setup_acpi_battery: %s\n",
+						bf->state ? bf->state : "no state");
 		return TRUE;
 		}
 	g_free(info);
@@ -1458,6 +1469,8 @@ setup_ac_adapter(gchar **state, gchar *ac)
 	if (!path)
 		path = get_acpi_battery_file(ACPI_AC_ADAPTOR_DIR, ac, "status");
 	*state = path;
+	if (_GK.debug_level & DEBUG_BATTERY)
+		printf("setup_ac_adaptor: %s\n", path ? path : "no state");
 
 	return path ? TRUE : FALSE;
 	}
@@ -1525,11 +1538,13 @@ acpi_battery_data(BatteryFile *bf)
 	gint			cur_cap = 0, cur_rate = 0;
 	extern gint		gkrellm_battery_full_cap_fallback(void);
 
-	if (!bf->got_info)	/* get battery capacity */
+	if (!bf->got_full_cap)	/* get battery capacity */
 		{
+		if (_GK.debug_level & DEBUG_BATTERY)
+			printf("getting full capacity: %s\n", bf->info);
 		if ((f = fopen(bf->info, "r")) == NULL)
 			return FALSE;
-		bf->got_info = TRUE;
+		bf->got_full_cap = TRUE;
 		while (fgets_lower_case(buf, sizeof(buf), f))
 			{
 			/* present:					{yes|no}
@@ -1546,13 +1561,20 @@ acpi_battery_data(BatteryFile *bf)
 			|  battery type:			4
 			|  OEM info:				XXXX
 			*/
-			sscanf(buf, "design capacity: %d", &bf->full_cap);
-			sscanf(buf, "last full capacity: %d", &bf->full_cap);
+			if (sscanf(buf, "design capacity: %d", &bf->full_cap) == 1)
+				if (_GK.debug_level & DEBUG_BATTERY)
+					printf("%s: %d <- %s", bf->info, bf->full_cap, buf);
+			if (sscanf(buf, "last full capacity: %d", &bf->full_cap) == 1)
+				if (_GK.debug_level & DEBUG_BATTERY)
+					printf("%s: %d <- %s", bf->info, bf->full_cap, buf);
 			}
 		fclose(f);
 		}
 	if (bf->full_cap == 0)
+		{
 		bf->full_cap = gkrellm_battery_full_cap_fallback();
+		bf->full_cap_bug = TRUE;
+		}
 
 	on_line = FALSE;
 
@@ -1598,7 +1620,11 @@ acpi_battery_data(BatteryFile *bf)
 			continue;
 			}
 		if (sscanf(buf, "remaining capacity: %d", &cur_cap) == 1)
+			{
+			if (_GK.debug_level & DEBUG_BATTERY)
+				printf("%s: %d <- %s", bf->state, cur_cap, buf);
 			continue;
+			}
 		if (sscanf(buf, "present rate: %d", &cur_rate) == 1)
 			continue;
 		if (   sscanf(buf, "present: %s", s1) == 1
@@ -1608,6 +1634,12 @@ acpi_battery_data(BatteryFile *bf)
 		}
 	fclose(f);
 
+	if (_GK.debug_level & DEBUG_BATTERY)
+		printf(
+			"Battery: on_line=%d charging=%d have_charging=%d state_file=%d\n",
+			on_line, charging, have_charging_state,
+			acpi_ac_state_file ? TRUE : FALSE);
+
 	if (!acpi_ac_state_file && charging)	/* Assumption for buggy ACPI */
 		on_line = TRUE;
 
@@ -1618,12 +1650,22 @@ acpi_battery_data(BatteryFile *bf)
 	if (percent > 100)
 		{
 		percent = 100;
-		bf->full_cap = cur_cap;
+		if (bf->full_cap_bug)
+			bf->full_cap = cur_cap;
+		else
+			bf->got_full_cap = FALSE;			/* reread full_cap */
 		}
 
 	if (cur_rate > 0)
 		time_left = 60 * cur_cap / cur_rate;
 
+	if (_GK.debug_level & DEBUG_BATTERY)
+		{
+		printf("Battery %d: percent=%d time_left=%d cur_cap=%d full_cap=%d\n",
+				bf->id, percent, time_left, cur_cap, bf->full_cap);
+		printf("            available=%d on_line=%d charging=%d fc_bug=%d\n",
+			available, on_line, charging, bf->full_cap_bug);
+		}
 	gkrellm_battery_assign_data(bf->id, available, on_line, charging,
 				percent, time_left);
 	return TRUE;
@@ -1821,6 +1863,7 @@ gkrellm_sys_uptime_init(void)
 /* Sensor monitor interface */
 /* ------- Linux ------------------------------------------------------- */
 
+
 #define	THERMAL_ZONE_DIR	"/proc/acpi/thermal_zone"
 #define	THERMAL_DIR			"/proc/acpi/thermal"
 #define	SENSORS_DIR			"/proc/sys/dev/sensors"
@@ -1829,6 +1872,7 @@ gkrellm_sys_uptime_init(void)
 #define	LM_SENSOR				0
 #define	THERMAL_SENSOR			1
 #define	THERMAL_ZONE_SENSOR		2
+#define	HDDTEMP_SENSOR			3
 
 typedef struct
 	{
@@ -1956,11 +2000,137 @@ static VoltDefault	voltdefault9[] =
 	};
 
 
+typedef struct
+	{
+	gchar	*device;
+	gfloat	value;
+	gchar	unit;
+	}
+	Hddtemp;
+
+GList	*hddtemp_list;
+
+static Hddtemp *
+hddtemp_lookup(gchar *device)
+	{
+	GList	*list;
+	Hddtemp	*hdd;
+
+	for (list = hddtemp_list; list; list = list->next)
+		{
+		hdd = (Hddtemp *) list->data;
+		if (!strcmp(device, hdd->device))
+			return hdd;
+		}
+	return NULL;
+	}
+
+static void
+hddtemp_daemon_read(void)
+	{
+	struct sockaddr_in	sin;
+	struct hostent		*he;
+	gpointer			sr;
+	Hddtemp				*hdd;
+	gchar				**argv, **info, *id_name, *default_label;
+	gchar				buf[256], sep;
+	gint				fd, n, j;
+	static GString		*hddtemp_gstring;
+
+	if (!hddtemp_gstring)
+		hddtemp_gstring = g_string_new("");
+	hddtemp_gstring = g_string_truncate(hddtemp_gstring, 0);
+
+	if ((he = gethostbyname("127.0.0.1")) == NULL)
+		return;
+	if ((fd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
+		return;
+
+	memset(&sin, 0, sizeof(struct sockaddr_in));
+	memcpy(&(sin.sin_addr.s_addr), he->h_addr, he->h_length);
+	sin.sin_family = AF_INET;
+	sin.sin_port = htons(7634);
+
+	if (connect(fd, (struct sockaddr *) &sin, (socklen_t) sizeof(sin)) == -1)
+		{
+		close(fd);
+		return;
+		}
+
+	while ((n = read(fd, buf, sizeof(buf) - 1)) > 0)
+		{
+		buf[n] = '\0';
+		hddtemp_gstring = g_string_append(hddtemp_gstring, buf);
+		}
+	close(fd);
+
+	if (_GK.debug_level & DEBUG_SENSORS)
+		printf("hddtemp_daemon_read: %s\n", hddtemp_gstring->str);
+
+	sep = hddtemp_gstring->str[0];
+	if (sep == '\0')
+		return;
+	sprintf(buf, "%c%c", sep, sep);
+
+	argv = g_strsplit(hddtemp_gstring->str + 1, buf, 20);
+	buf[1] = '\0';
+	for (n = 0; argv[n] != NULL; ++n)
+		{
+		info = g_strsplit(argv[n], buf, 4);
+		for (j = 0; info[j] != NULL; ++j)
+			;
+		if (j < 4)
+			{
+			g_strfreev(info);
+			continue;
+			}
+		if ((hdd = hddtemp_lookup(info[0])) == NULL)
+			{
+			hdd = g_new0(Hddtemp, 1);
+			hddtemp_list = g_list_append(hddtemp_list, hdd);
+			hdd->device = g_strdup(info[0]);
+			default_label = strrchr(hdd->device, '/');
+			if (default_label)
+				++default_label;
+			else
+				default_label = hdd->device;
+			id_name = g_strdup_printf("hddtemp/%s", default_label);
+			sr = gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
+						hdd->device, id_name,
+						0, 0, HDDTEMP_SENSOR,
+						1.0, 0.0, NULL, default_label);
+			gkrellm_sensors_set_group(sr, SENSOR_GROUP_DISK);
+			g_free(id_name);
+			}
+		hdd->value = atof(info[2]);
+		if (*info[3] == 'F')
+			hdd->value = (hdd->value - 32.0) / 1.8;
+		g_strfreev(info);
+		}
+	g_strfreev(argv);
+	}
+
+static void
+hddtemp_check(void)
+	{
+	static gint	check_time = -1;
+
+	/* hddtemp docs say shouldn't check more than once per minute.
+	*/
+	if (check_time < _GK.time_now)
+		{
+		hddtemp_daemon_read();
+		check_time = _GK.time_now + 60;
+		}
+	}
+
+
 gboolean
 gkrellm_sys_sensors_get_temperature(gchar *sensor_path, gint id,
 		gint iodev, gint interface, gfloat *temp)
 	{
 	FILE		*f;
+	Hddtemp		*hdd;
 	gchar		buf[64], units[32];
 	gint		n;
 	gfloat		T, t[5];
@@ -1987,7 +2157,18 @@ gkrellm_sys_sensors_get_temperature(gchar *sensor_path, gint id,
 			}
 		return result;
 		}
-	
+
+	if (interface == HDDTEMP_SENSOR)
+		{
+		hddtemp_check();
+		if ((hdd = hddtemp_lookup(sensor_path)) != NULL)
+			{
+			*temp = hdd->value;
+			result = TRUE;
+			}
+		return result;
+		}
+
 	if ((f = fopen(sensor_path, "r")) == NULL)
 		return FALSE;
 	fgets(buf, sizeof(buf), f);
@@ -2001,7 +2182,7 @@ gkrellm_sys_sensors_get_temperature(gchar *sensor_path, gint id,
 	T = t[n - 1];
 
 	if (have_sysfs_sensors)
-		T /= 100.0;
+		T /= 1000.0;
 	else if (T > 254.0)		/* Bogus read from BIOS if CHAR_MAX */
 		return FALSE;
 
@@ -2167,7 +2348,7 @@ sysfs_sensors_init(void)
 	gint			type, voltdefaultsize;
 	gfloat			factor, offset;
 	gchar			*name, *bus_name, *default_label, *vref,
-					*id_name,  *chip_name, *s, *sensor_path;
+					*id_name,  *chip_name, *s, *d, *sensor_path;
 	gchar			path[256], buf[256];
 
 	if (!have_sysfs)
@@ -2192,13 +2373,16 @@ sysfs_sensors_init(void)
 		get_volt_default(chip_name, &voltdefault, &voltdefaultsize);
 		while ((name = (gchar *) g_dir_read_name(chip_dir)) != NULL)
 			{
-			if (   (s = strstr(name, "_input")) == NULL
-				|| s - name > 4 || (strlen(s) > 6 && !isdigit(*(s + 6)))
-			   )
+			snprintf(buf, sizeof(buf), "%s", name);
+			if ((s = strstr(buf, "_input")) == NULL || s - buf > 6)
 				continue;
-			strncpy(buf, name, s - name);
-			strcpy(&buf[s - name], s + 6);
-			id = atoi(s + 6);
+			d = s + 6;		/* Can have xxxN_input, xxx_inputN, or xxx_input */
+			while (isdigit(*d))
+				*s++ = *d++;
+			*s = '\0';
+			while (isdigit(*(s-1)))
+				--s;
+			id = atoi(s);
 
 			if (!strncmp(buf, "temp", 4))
 				type = SENSOR_TEMPERATURE;
@@ -2312,6 +2496,8 @@ gkrellm_sys_sensors_init(void)
 		g_dir_close(dir);
 		}
 
+	hddtemp_check();
+
 	if ((dir = g_dir_open(SENSORS_DIR, 0, NULL)) == NULL)
 		{
 		sysfs_sensors_init();
