@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -54,27 +54,53 @@ gkrellm_sys_main_cleanup(void)
 #include <sys/sysctl.h>
 #include <sys/sched.h>
 
+static gint ncpus;
+
+static gint get_ncpus(void);
+
 void
 gkrellm_sys_cpu_read_data(void)
 {
    static int mib[] = { CTL_KERN, KERN_CP_TIME };
-   u_int64_t cp_time[CPUSTATES];
-   int len = sizeof(cp_time);
+   u_int64_t cp_time[ncpus][CPUSTATES];
+   int len, n;
 
+   if (ncpus > 1) {
+       len = sizeof(cp_time[0]);
+       /* The sysctl() is magic -- it returns the aggregate if
+	  there's not enough room for all CPU's. */
+       if (sysctl(mib, 2, cp_time, &len, NULL, 0) == 0)
+	   gkrellm_cpu_assign_composite_data(cp_time[0][CP_USER],
+	       cp_time[0][CP_NICE], cp_time[0][CP_SYS], cp_time[0][CP_IDLE]);
+   }
+
+   len = sizeof(cp_time);
    if (sysctl(mib, 2, cp_time, &len, NULL, 0) < 0) return;
-
-   /* Currently, SMP is not supported */
-	gkrellm_cpu_assign_data(0, cp_time[CP_USER], cp_time[CP_NICE],
-				cp_time[CP_SYS], cp_time[CP_IDLE]);
+   for (n = 0; n < ncpus; n++)
+	gkrellm_cpu_assign_data(n, cp_time[n][CP_USER],
+	   cp_time[n][CP_NICE], cp_time[n][CP_SYS], cp_time[n][CP_IDLE]);
 }
 
 gboolean
 gkrellm_sys_cpu_init(void)
     {
-	gkrellm_cpu_set_number_of_cpus(1);
+	ncpus = get_ncpus();
+	gkrellm_cpu_set_number_of_cpus(ncpus);
 	return TRUE;
 	}
 
+static gint
+get_ncpus(void)
+{
+	static int mib[] = { CTL_HW, HW_NCPU };
+	int ncpus;
+	int len = sizeof(int);
+
+	if (sysctl(mib, 2, &ncpus, &len, NULL, 0) < 0)
+		return 1;
+	else
+		return ncpus;
+}
 
 
 /* ===================================================================== */
